# @summary File LNK
#
# Defines a .LNK shortcut
#
# @param description
#   Descriptive comment to use for the shortcut
#   Example: 'This is an example comment'
#
# @param ensure
#   Desired state of the shortcut
#   Example: 'absent'
#
# @param file_name
#   Assigned text, including the file extension, that should be used as the name of the file
#   Example: 'OneToRuleThemAll.lnk'
#
# @param hot_key
#   Denotes the key combination to assign to the shortcut
#   Key combo execution will only work if the shortcut resides in certain locations (such as Desktop, Start Menu, etc..)
#   Example: 'CTRL+ALT+8'
#
# @param icon_file
#   Full path to the file containing the icon to use for the shortcut
#   Example: 'C:\Windows\HelpPane.exe'
#
# @param icon_index
#   Denotes which icon entry within the icon_file item to use for the shortcut
#   Example: 0
#
# @param log_output
#   Determines additional logging output (typically only used for troubleshooting) during a Puppet agent run
#   Example: True
#
# @param parent_folder
#   Full path to the parent folder that will house the shortcut file
#   Example: 'C:\Users\Public\Desktop'
#
# @param set_run_as_administrator
#   Determines if the shortcut should be run with administrator rights
#   Example: True
#
# @param target_path
#   Path to the item to launch when the shortcut is triggered
#   Example: 'C:\\Windows\\notepad.exe'
#
# @param target_arguments
#   Arguments to supply to the target file when the shorcut is triggered
#   Example: '-option123'
#
# @param window_style
#   Denotes what type of window to use when the shortcut is triggered
#   1=normal, 3=maximized, 7=minimized
#   Example: '3'
#
# @param working_directory
#   Path to use for the working directory when the shortcut is triggered
#   Example: 'C:\\Windows'
#
define file_lnk (
  String $file_name,
  String $parent_folder,
  String $target_path,
  Optional[String] $description = '',
  Optional[Enum['present', 'absent']] $ensure = 'present',
  Optional[String] $hot_key = '',
  Optional[String] $icon_file = $target_path,
  Optional[Integer] $icon_index = 0,
  Optional[Boolean] $log_output = false,
  Optional[Boolean] $set_run_as_administrator = false,
  Optional[String] $target_arguments = '',
  Optional[Enum['1', '3', '7']] $window_style = '1',
  Optional[String] $working_directory = 'C:\\Windows',
) {
  if ($log_output) {
    notify {"${module_name} - ${title} - file_name: '${file_name}'":}
    notify {"${module_name} - ${title} - ensure: '${ensure}'":}
    notify {"${module_name} - ${title} - parent_folder: '${parent_folder}'":}
  }
  if ($ensure == 'absent') {
    tidy { "Remove ${title} LNK shortcut":
      path    => $parent_folder,
      age     => '0',
      recurse => 1,
      matches => [ $file_name ],
    }
  } else {
    if ($log_output) {
      notify {"${module_name} - ${title} - target_path: '${target_path}'":}
      notify {"${module_name} - ${title} - description: '${description}'":}
      notify {"${module_name} - ${title} - hot_key: '${hot_key}'":}
      notify {"${module_name} - ${title} - icon_file: '${icon_file}'":}
      notify {"${module_name} - ${title} - icon_index: '${icon_index}'":}
      notify {"${module_name} - ${title} - set_run_as_administrator: '${set_run_as_administrator}'":}
      notify {"${module_name} - ${title} - window_style: '${window_style}'":}
      notify {"${module_name} - ${title} - working_directory: '${working_directory}'":}
    }
    exec { "Set '${title}' LNK shortcut":
      provider  => powershell,
      command   => template(
        "${module_name}/helpers.ps1.erb",
        "${module_name}/command.ps1.erb"
      ),
      onlyif    => template(
        "${module_name}/helpers.ps1.erb",
        "${module_name}/onlyif.ps1.erb"
      ),
      logoutput => $log_output
    }
  }
}
